@@ -32,7 +32,7 @@ sub format_link
 		}
 	}
 
-	return "[$text]($page)";
+	return "[$text](https://collectd.org/wiki/index.php/$page)";
 }
 
 while (<>)
@@ -51,6 +51,8 @@ while (<>)
 	$line =~ s#\[\[([^|]+)\|([^\]]+)\]\]#format_link($1, $2)#ge;
 	$line =~ s#\[\[([^|]+)\]\]#format_link($1)#ge;
 
+	$line =~ s#('')?Ruben Kerkhof('')?#\@rubenk#g;
+
 	$line =~ s#'''(.*?)'''#**$1**#g;
 	$line =~ s#''(.*?)''#*$1*#g;
 	$line =~ s#<code>(.*?)</code>#`$1`#g;
