@@ -39,18 +39,12 @@
  */
 
 #include "collectd.h"
+
 #include "common.h"
 #include "plugin.h"
 
-#include "configfile.h"
-
 #include <stddef.h>
 
-#if HAVE_LIBPTHREAD
-# include <pthread.h>
-#endif
-
-#include <sys/socket.h>
 #include <sys/un.h>
 #include <sys/select.h>
 
@@ -220,7 +214,7 @@ static int email_config (const char *key, const char *value)
 static void type_list_incr (type_list_t *list, char *name, int incr)
 {
 	if (NULL == list->head) {
-		list->head = (type_t *)smalloc (sizeof (type_t));
+		list->head = smalloc (sizeof (*list->head));
 
 		list->head->name  = sstrdup (name);
 		list->head->value = incr;
@@ -237,7 +231,7 @@ static void type_list_incr (type_list_t *list, char *name, int incr)
 		}
 
 		if (NULL == ptr) {
-			list->tail->next = (type_t *)smalloc (sizeof (type_t));
+			list->tail->next = smalloc (sizeof (*list->tail->next));
 			list->tail = list->tail->next;
 
 			list->tail->name  = sstrdup (name);
@@ -390,8 +384,8 @@ static void *open_connection (void __attribute__((unused)) *arg)
 {
 	struct sockaddr_un addr;
 
-	char *path  = (NULL == sock_file) ? SOCK_PATH : sock_file;
-	char *group = (NULL == sock_group) ? COLLECTD_GRP_NAME : sock_group;
+	const char *path  = (NULL == sock_file) ? SOCK_PATH : sock_file;
+	const char *group = (NULL == sock_group) ? COLLECTD_GRP_NAME : sock_group;
 
 	/* create UNIX socket */
 	errno = 0;
@@ -469,9 +463,6 @@ static void *open_connection (void __attribute__((unused)) *arg)
 	}
 
 	{ /* initialize collector threads */
-		int i   = 0;
-		int err = 0;
-
 		pthread_attr_t ptattr;
 
 		conns.head = NULL;
@@ -483,16 +474,16 @@ static void *open_connection (void __attribute__((unused)) *arg)
 		available_collectors = max_conns;
 
 		collectors =
-			(collector_t **)smalloc (max_conns * sizeof (collector_t *));
+			smalloc (max_conns * sizeof (*collectors));
 
-		for (i = 0; i < max_conns; ++i) {
-			collectors[i] = (collector_t *)smalloc (sizeof (collector_t));
+		for (int i = 0; i < max_conns; ++i) {
+			collectors[i] = smalloc (sizeof (*collectors[i]));
 			collectors[i]->socket = NULL;
 
-			if (0 != (err = plugin_thread_create (&collectors[i]->thread,
-							&ptattr, collect, collectors[i]))) {
+			if (plugin_thread_create (&collectors[i]->thread,
+							&ptattr, collect, collectors[i]) != 0) {
 				char errbuf[1024];
-				log_err ("pthread_create() failed: %s",
+				log_err ("plugin_thread_create() failed: %s",
 						sstrerror (errno, errbuf, sizeof (errbuf)));
 				collectors[i]->thread = (pthread_t) 0;
 			}
@@ -538,13 +529,12 @@ static void *open_connection (void __attribute__((unused)) *arg)
 			break;
 		}
 
-		connection = malloc (sizeof (*connection));
+		connection = calloc (1, sizeof (*connection));
 		if (connection == NULL)
 		{
 			close (remote);
 			continue;
 		}
-		memset (connection, 0, sizeof (*connection));
 
 		connection->socket = fdopen (remote, "r");
 		connection->next   = NULL;
@@ -577,13 +567,11 @@ static void *open_connection (void __attribute__((unused)) *arg)
 
 static int email_init (void)
 {
-	int err = 0;
-
-	if (0 != (err = plugin_thread_create (&connector, NULL,
-				open_connection, NULL))) {
+	if (plugin_thread_create (&connector, NULL,
+				open_connection, NULL) != 0) {
 		char errbuf[1024];
 		disabled = 1;
-		log_err ("pthread_create() failed: %s",
+		log_err ("plugin_thread_create() failed: %s",
 				sstrerror (errno, errbuf, sizeof (errbuf)));
 		return (-1);
 	}
@@ -612,8 +600,6 @@ static void type_list_free (type_list_t *t)
 
 static int email_shutdown (void)
 {
-	int i = 0;
-
 	if (connector != ((pthread_t) 0)) {
 		pthread_kill (connector, SIGTERM);
 		connector = (pthread_t) 0;
@@ -630,7 +616,7 @@ static int email_shutdown (void)
 	available_collectors = 0;
 
 	if (collectors != NULL) {
-		for (i = 0; i < max_conns; ++i) {
+		for (int i = 0; i < max_conns; ++i) {
 			if (collectors[i] == NULL)
 				continue;
 
@@ -688,15 +674,12 @@ static void email_submit (const char *type, const char *type_instance, gauge_t v
  * after they have been copied to l2. */
 static void copy_type_list (type_list_t *l1, type_list_t *l2)
 {
-	type_t *ptr1;
-	type_t *ptr2;
-
 	type_t *last = NULL;
 
-	for (ptr1 = l1->head, ptr2 = l2->head; NULL != ptr1;
+	for (type_t *ptr1 = l1->head, *ptr2 = l2->head; NULL != ptr1;
 			ptr1 = ptr1->next, last = ptr2, ptr2 = ptr2->next) {
 		if (NULL == ptr2) {
-			ptr2 = (type_t *)smalloc (sizeof (type_t));
+			ptr2 = smalloc (sizeof (*ptr2));
 			ptr2->name = NULL;
 			ptr2->next = NULL;
 
@@ -722,8 +705,6 @@ static void copy_type_list (type_list_t *l1, type_list_t *l2)
 
 static int email_read (void)
 {
-	type_t *ptr;
-
 	double score_old;
 	int score_count_old;
 
@@ -737,7 +718,7 @@ static int email_read (void)
 
 	pthread_mutex_unlock (&count_mutex);
 
-	for (ptr = list_count_copy.head; NULL != ptr; ptr = ptr->next) {
+	for (type_t *ptr = list_count_copy.head; NULL != ptr; ptr = ptr->next) {
 		email_submit ("email_count", ptr->name, ptr->value);
 	}
 
@@ -748,7 +729,7 @@ static int email_read (void)
 
 	pthread_mutex_unlock (&size_mutex);
 
-	for (ptr = list_size_copy.head; NULL != ptr; ptr = ptr->next) {
+	for (type_t *ptr = list_size_copy.head; NULL != ptr; ptr = ptr->next) {
 		email_submit ("email_size", ptr->name, ptr->value);
 	}
 
@@ -772,7 +753,7 @@ static int email_read (void)
 
 	pthread_mutex_unlock (&check_mutex);
 
-	for (ptr = list_check_copy.head; NULL != ptr; ptr = ptr->next)
+	for (type_t *ptr = list_check_copy.head; NULL != ptr; ptr = ptr->next)
 		email_submit ("spam_check", ptr->name, ptr->value);
 
 	return (0);
