@@ -28,9 +28,9 @@
  **/
 
 #include "collectd.h"
+
 #include "common.h"
 #include "plugin.h"
-#include "configfile.h"
 
 #ifdef HAVE_MYSQL_H
 #include <mysql.h>
@@ -46,6 +46,14 @@ struct mysql_database_s /* {{{ */
 	char *user;
 	char *pass;
 	char *database;
+
+	/* mysql_ssl_set params */
+	char *key;
+	char *cert;
+	char *ca;
+	char *capath;
+	char *cipher;
+
 	char *socket;
 	int   port;
 	int   timeout;
@@ -53,6 +61,7 @@ struct mysql_database_s /* {{{ */
 	_Bool master_stats;
 	_Bool slave_stats;
 	_Bool innodb_stats;
+	_Bool wsrep_stats;
 
 	_Bool slave_notif;
 	_Bool slave_io_running;
@@ -65,16 +74,13 @@ typedef struct mysql_database_s mysql_database_t; /* }}} */
 
 static int mysql_read (user_data_t *ud);
 
-void mysql_read_default_options(struct st_mysql_options *options,
-		const char *filename,const char *group);
-
 static void mysql_database_free (void *arg) /* {{{ */
 {
 	mysql_database_t *db;
 
 	DEBUG ("mysql plugin: mysql_database_free (arg = %p);", arg);
 
-	db = (mysql_database_t *) arg;
+	db = arg;
 
 	if (db == NULL)
 		return;
@@ -89,6 +95,11 @@ static void mysql_database_free (void *arg) /* {{{ */
 	sfree (db->socket);
 	sfree (db->instance);
 	sfree (db->database);
+	sfree (db->key);
+	sfree (db->cert);
+	sfree (db->ca);
+	sfree (db->capath);
+	sfree (db->cipher);
 	sfree (db);
 } /* }}} void mysql_database_free */
 
@@ -106,7 +117,6 @@ static int mysql_config_database (oconfig_item_t *ci) /* {{{ */
 {
 	mysql_database_t *db;
 	int status = 0;
-	int i;
 
 	if ((ci->values_num != 1)
 	    || (ci->values[0].type != OCONFIG_TYPE_STRING))
@@ -116,13 +126,12 @@ static int mysql_config_database (oconfig_item_t *ci) /* {{{ */
 		return (-1);
 	}
 
-	db = (mysql_database_t *) malloc (sizeof (*db));
+	db = calloc (1, sizeof (*db));
 	if (db == NULL)
 	{
-		ERROR ("mysql plugin: malloc failed.");
+		ERROR ("mysql plugin: calloc failed.");
 		return (-1);
 	}
-	memset (db, 0, sizeof (*db));
 
 	/* initialize all the pointers */
 	db->alias    = NULL;
@@ -130,6 +139,12 @@ static int mysql_config_database (oconfig_item_t *ci) /* {{{ */
 	db->user     = NULL;
 	db->pass     = NULL;
 	db->database = NULL;
+	db->key      = NULL;
+	db->cert     = NULL;
+	db->ca       = NULL;
+	db->capath   = NULL;
+	db->cipher   = NULL;
+
 	db->socket   = NULL;
 	db->con      = NULL;
 	db->timeout  = 0;
@@ -147,7 +162,7 @@ static int mysql_config_database (oconfig_item_t *ci) /* {{{ */
 	assert (db->instance != NULL);
 
 	/* Fill the `mysql_database_t' structure.. */
-	for (i = 0; i < ci->children_num; i++)
+	for (int i = 0; i < ci->children_num; i++)
 	{
 		oconfig_item_t *child = ci->children + i;
 
@@ -172,6 +187,16 @@ static int mysql_config_database (oconfig_item_t *ci) /* {{{ */
 			status = cf_util_get_string (child, &db->socket);
 		else if (strcasecmp ("Database", child->key) == 0)
 			status = cf_util_get_string (child, &db->database);
+		else if (strcasecmp ("SSLKey", child->key) == 0)
+			status = cf_util_get_string (child, &db->key);
+		else if (strcasecmp ("SSLCert", child->key) == 0)
+			status = cf_util_get_string (child, &db->cert);
+		else if (strcasecmp ("SSLCA", child->key) == 0)
+			status = cf_util_get_string (child, &db->ca);
+		else if (strcasecmp ("SSLCAPath", child->key) == 0)
+			status = cf_util_get_string (child, &db->capath);
+		else if (strcasecmp ("SSLCipher", child->key) == 0)
+			status = cf_util_get_string (child, &db->cipher);
 		else if (strcasecmp ("ConnectTimeout", child->key) == 0)
 			status = cf_util_get_int (child, &db->timeout);
 		else if (strcasecmp ("MasterStats", child->key) == 0)
@@ -182,6 +207,8 @@ static int mysql_config_database (oconfig_item_t *ci) /* {{{ */
 			status = cf_util_get_boolean (child, &db->slave_notif);
 		else if (strcasecmp ("InnodbStats", child->key) == 0)
 			status = cf_util_get_boolean (child, &db->innodb_stats);
+		else if (strcasecmp ("WsrepStats", child->key) == 0)
+			status = cf_util_get_boolean (child, &db->wsrep_stats);
 		else
 		{
 			WARNING ("mysql plugin: Option `%s' not allowed here.", child->key);
@@ -195,25 +222,25 @@ static int mysql_config_database (oconfig_item_t *ci) /* {{{ */
 	/* If all went well, register this database for reading */
 	if (status == 0)
 	{
-		user_data_t ud;
 		char cb_name[DATA_MAX_NAME_LEN];
 
 		DEBUG ("mysql plugin: Registering new read callback: %s",
 				(db->database != NULL) ? db->database : "<default>");
 
-		memset (&ud, 0, sizeof (ud));
-		ud.data = (void *) db;
-		ud.free_func = mysql_database_free;
-
 		if (db->instance != NULL)
 			ssnprintf (cb_name, sizeof (cb_name), "mysql-%s",
 					db->instance);
 		else
 			sstrncpy (cb_name, "mysql", sizeof (cb_name));
 
+		user_data_t ud = {
+			.data = db,
+			.free_func = mysql_database_free
+		};
+
 		plugin_register_complex_read (/* group = */ NULL, cb_name,
 					      mysql_read,
-					      /* interval = */ NULL, &ud);
+					      /* interval = */ 0, &ud);
 	}
 	else
 	{
@@ -226,13 +253,11 @@ static int mysql_config_database (oconfig_item_t *ci) /* {{{ */
 
 static int mysql_config (oconfig_item_t *ci) /* {{{ */
 {
-	int i;
-
 	if (ci == NULL)
 		return (EINVAL);
 
 	/* Fill the `mysql_database_t' structure.. */
-	for (i = 0; i < ci->children_num; i++)
+	for (int i = 0; i < ci->children_num; i++)
 	{
 		oconfig_item_t *child = ci->children + i;
 
@@ -250,6 +275,8 @@ static int mysql_config (oconfig_item_t *ci) /* {{{ */
 
 static MYSQL *getconnection (mysql_database_t *db)
 {
+	const char *cipher;
+
 	if (db->is_connected)
 	{
 		int status;
@@ -277,6 +304,8 @@ static MYSQL *getconnection (mysql_database_t *db)
 	/* Configure TCP connect timeout (default: 0) */
 	db->con->options.connect_timeout = db->timeout;
 
+	mysql_ssl_set (db->con, db->key, db->cert, db->ca, db->capath, db->cipher);
+
 	if (mysql_real_connect (db->con, db->host, db->user, db->pass,
 				db->database, db->port, db->socket, 0) == NULL)
 	{
@@ -288,10 +317,14 @@ static MYSQL *getconnection (mysql_database_t *db)
 		return (NULL);
 	}
 
+	cipher = mysql_get_ssl_cipher (db->con);
+
 	INFO ("mysql plugin: Successfully connected to database %s "
-			"at server %s (server version: %s, protocol version: %d)",
+			"at server %s with cipher %s "
+			"(server version: %s, protocol version: %d) ",
 			(db->database != NULL) ? db->database : "<none>",
 			mysql_get_host_info (db->con),
+			(cipher != NULL) ?  cipher : "<none>",
 			mysql_get_server_info (db->con),
 			mysql_get_proto_info (db->con));
 
@@ -403,8 +436,8 @@ static int mysql_read_master_stats (mysql_database_t *db, MYSQL *con)
 	MYSQL_RES *res;
 	MYSQL_ROW  row;
 
-	char *query;
-	int   field_num;
+	const char *query;
+	int         field_num;
 	unsigned long long position;
 
 	query = "SHOW MASTER STATUS";
@@ -449,8 +482,8 @@ static int mysql_read_slave_stats (mysql_database_t *db, MYSQL *con)
 	MYSQL_RES *res;
 	MYSQL_ROW  row;
 
-	char *query;
-	int   field_num;
+	const char *query;
+	int         field_num;
 
 	/* WTF? libmysqlclient does not seem to provide any means to
 	 * translate a column name to a column index ... :-/ */
@@ -572,84 +605,53 @@ static int mysql_read_innodb_stats (mysql_database_t *db, MYSQL *con)
 	MYSQL_RES *res;
 	MYSQL_ROW  row;
 
-	char *query;
-    struct {
-        char *key;
-        char *type;
-        int ds_type;
-    } metrics[] = {
-        { "metadata_mem_pool_size",         "bytes",        DS_TYPE_GAUGE },
-        { "lock_deadlocks",                 "mysql_locks",  DS_TYPE_DERIVE },
-        { "lock_timeouts",                  "mysql_locks",  DS_TYPE_DERIVE },
-        { "lock_row_lock_current_waits",    "mysql_locks",  DS_TYPE_DERIVE },
-        { "buffer_pool_size",               "bytes",        DS_TYPE_GAUGE },
+	const char *query;
+	struct {
+		const char *key;
+		const char *type;
+		int ds_type;
+	} metrics[] = {
+		{ "metadata_mem_pool_size",          "bytes",        DS_TYPE_GAUGE },
+		{ "lock_deadlocks",                  "mysql_locks",  DS_TYPE_DERIVE },
+		{ "lock_timeouts",                   "mysql_locks",  DS_TYPE_DERIVE },
+		{ "lock_row_lock_current_waits",     "mysql_locks",  DS_TYPE_DERIVE },
+		{ "buffer_pool_size",                "bytes",        DS_TYPE_GAUGE },
 
-        { "buffer_pool_reads",              "operations",   DS_TYPE_DERIVE },
-        { "buffer_pool_read_requests",      "operations",   DS_TYPE_DERIVE },
-        { "buffer_pool_write_requests",     "operations",   DS_TYPE_DERIVE },
-        { "buffer_pool_wait_free",          "operations",   DS_TYPE_DERIVE },
-        { "buffer_pool_read_ahead",         "operations",   DS_TYPE_DERIVE },
-        { "buffer_pool_read_ahead_evicted", "operations",   DS_TYPE_DERIVE },
+		{ "os_log_bytes_written",            "operations",   DS_TYPE_DERIVE },
+		{ "os_log_pending_fsyncs",           "operations",   DS_TYPE_DERIVE },
+		{ "os_log_pending_writes",           "operations",   DS_TYPE_DERIVE },
 
-        { "buffer_pool_pages_total",        "gauge",        DS_TYPE_GAUGE },
-        { "buffer_pool_pages_misc",         "gauge",        DS_TYPE_GAUGE },
-        { "buffer_pool_pages_data",         "gauge",        DS_TYPE_GAUGE },
-        { "buffer_pool_bytes_data",         "gauge",        DS_TYPE_GAUGE },
-        { "buffer_pool_pages_dirty",        "gauge",        DS_TYPE_GAUGE },
-        { "buffer_pool_bytes_dirty",        "gauge",        DS_TYPE_GAUGE },
-        { "buffer_pool_pages_free",         "gauge",        DS_TYPE_GAUGE },
+		{ "trx_rseg_history_len",            "gauge",        DS_TYPE_GAUGE },
 
-        { "buffer_pages_created",           "operations",   DS_TYPE_DERIVE },
-        { "buffer_pages_written",           "operations",   DS_TYPE_DERIVE },
-        { "buffer_pages_read",              "operations",   DS_TYPE_DERIVE },
-        { "buffer_data_reads",              "operations",   DS_TYPE_DERIVE },
-        { "buffer_data_written",            "operations",   DS_TYPE_DERIVE },
+		{ "adaptive_hash_searches",          "operations",   DS_TYPE_DERIVE },
 
-        { "os_data_reads",                  "operations",   DS_TYPE_DERIVE },
-        { "os_data_writes",                 "operations",   DS_TYPE_DERIVE },
-        { "os_data_fsyncs",                 "operations",   DS_TYPE_DERIVE },
-        { "os_log_bytes_written",           "operations",   DS_TYPE_DERIVE },
-        { "os_log_fsyncs",                  "operations",   DS_TYPE_DERIVE },
-        { "os_log_pending_fsyncs",          "operations",   DS_TYPE_DERIVE },
-        { "os_log_pending_writes",          "operations",   DS_TYPE_DERIVE },
+		{ "file_num_open_files",             "gauge",        DS_TYPE_GAUGE },
 
-        { "trx_rseg_history_len",           "gauge",        DS_TYPE_GAUGE },
+		{ "ibuf_merges_insert",              "operations",   DS_TYPE_DERIVE },
+		{ "ibuf_merges_delete_mark",         "operations",   DS_TYPE_DERIVE },
+		{ "ibuf_merges_delete",              "operations",   DS_TYPE_DERIVE },
+		{ "ibuf_merges_discard_insert",      "operations",   DS_TYPE_DERIVE },
+		{ "ibuf_merges_discard_delete_mark", "operations",   DS_TYPE_DERIVE },
+		{ "ibuf_merges_discard_delete",      "operations",   DS_TYPE_DERIVE },
+		{ "ibuf_merges_discard_merges",      "operations",   DS_TYPE_DERIVE },
+		{ "ibuf_size",                       "bytes",        DS_TYPE_GAUGE },
 
-        { "log_waits",                      "operations",   DS_TYPE_DERIVE },
-        { "log_write_requests",             "operations",   DS_TYPE_DERIVE },
-        { "log_writes",                     "operations",   DS_TYPE_DERIVE },
-        { "adaptive_hash_searches",         "operations",   DS_TYPE_DERIVE },
+		{ "innodb_activity_count",           "gauge",        DS_TYPE_GAUGE },
 
-        { "file_num_open_files",            "gauge",        DS_TYPE_GAUGE },
+		{ "innodb_rwlock_s_spin_waits",      "operations",   DS_TYPE_DERIVE },
+		{ "innodb_rwlock_x_spin_waits",      "operations",   DS_TYPE_DERIVE },
+		{ "innodb_rwlock_s_spin_rounds",     "operations",   DS_TYPE_DERIVE },
+		{ "innodb_rwlock_x_spin_rounds",     "operations",   DS_TYPE_DERIVE },
+		{ "innodb_rwlock_s_os_waits",        "operations",   DS_TYPE_DERIVE },
+		{ "innodb_rwlock_x_os_waits",        "operations",   DS_TYPE_DERIVE },
 
-        { "ibuf_merges_insert",             "operations",   DS_TYPE_DERIVE },
-        { "ibuf_merges_delete_mark",        "operations",   DS_TYPE_DERIVE },
-        { "ibuf_merges_delete",             "operations",   DS_TYPE_DERIVE },
-        { "ibuf_merges_discard_insert",     "operations",   DS_TYPE_DERIVE },
-        { "ibuf_merges_discard_delete_mark","operations",   DS_TYPE_DERIVE },
-        { "ibuf_merges_discard_delete",     "operations",   DS_TYPE_DERIVE },
-        { "ibuf_merges_discard_merges",     "operations",   DS_TYPE_DERIVE },
-        { "ibuf_size",                      "bytes",        DS_TYPE_GAUGE },
+		{ "dml_reads",                       "operations",   DS_TYPE_DERIVE },
+		{ "dml_inserts",                     "operations",   DS_TYPE_DERIVE },
+		{ "dml_deletes",                     "operations",   DS_TYPE_DERIVE },
+		{ "dml_updates",                     "operations",   DS_TYPE_DERIVE },
 
-        { "innodb_activity_count",          "gauge",        DS_TYPE_GAUGE },
-        { "innodb_dblwr_writes",            "operations",   DS_TYPE_DERIVE },
-        { "innodb_dblwr_pages_written",     "operations",   DS_TYPE_DERIVE },
-        { "innodb_dblwr_page_size",         "gauge",        DS_TYPE_GAUGE },
-
-        { "innodb_rwlock_s_spin_waits",     "operations",   DS_TYPE_DERIVE },
-        { "innodb_rwlock_x_spin_waits",     "operations",   DS_TYPE_DERIVE },
-        { "innodb_rwlock_s_spin_rounds",    "operations",   DS_TYPE_DERIVE },
-        { "innodb_rwlock_x_spin_rounds",    "operations",   DS_TYPE_DERIVE },
-        { "innodb_rwlock_s_os_waits",       "operations",   DS_TYPE_DERIVE },
-        { "innodb_rwlock_x_os_waits",       "operations",   DS_TYPE_DERIVE },
-
-        { "dml_reads",                      "operations",   DS_TYPE_DERIVE },
-        { "dml_inserts",                    "operations",   DS_TYPE_DERIVE },
-        { "dml_deletes",                    "operations",   DS_TYPE_DERIVE },
-        { "dml_updates",                    "operations",   DS_TYPE_DERIVE },
-
-        { NULL,                     NULL,           0}
-    };
+		{ NULL,                              NULL,           0}
+	};
 
 	query = "SELECT name, count, type FROM information_schema.innodb_metrics WHERE status = 'enabled'";
 
@@ -659,45 +661,130 @@ static int mysql_read_innodb_stats (mysql_database_t *db, MYSQL *con)
 
 	while ((row = mysql_fetch_row (res)))
 	{
-        int i;
+		int i;
 		char *key;
 		unsigned long long val;
 
 		key = row[0];
 		val = atoll (row[1]);
 
-        for (i = 0;
-             metrics[i].key != NULL && strcmp(metrics[i].key, key) != 0;
-             i++)
-            ;
+		for (i = 0; metrics[i].key != NULL && strcmp(metrics[i].key, key) != 0; i++)
+			;
 
-        if (metrics[i].key == NULL)
-            continue;
+		if (metrics[i].key == NULL)
+			continue;
 
-        switch (metrics[i].ds_type) {
-        case DS_TYPE_COUNTER:
-            counter_submit(metrics[i].type, key, (counter_t)val, db);
-            break;
-        case DS_TYPE_GAUGE:
-            gauge_submit(metrics[i].type, key, (gauge_t)val, db);
-            break;
-        case DS_TYPE_DERIVE:
-            derive_submit(metrics[i].type, key, (derive_t)val, db);
-            break;
-        }
-    }
+		switch (metrics[i].ds_type) {
+			case DS_TYPE_COUNTER:
+				counter_submit(metrics[i].type, key, (counter_t)val, db);
+				break;
+			case DS_TYPE_GAUGE:
+				gauge_submit(metrics[i].type, key, (gauge_t)val, db);
+				break;
+			case DS_TYPE_DERIVE:
+				derive_submit(metrics[i].type, key, (derive_t)val, db);
+				break;
+		}
+	}
 
-    mysql_free_result(res);
-    return (0);
+	mysql_free_result(res);
+	return (0);
 }
 
+static int mysql_read_wsrep_stats (mysql_database_t *db, MYSQL *con)
+{
+	MYSQL_RES *res;
+	MYSQL_ROW  row;
+
+	const char *query;
+	struct {
+		const char *key;
+		const char *type;
+		int ds_type;
+	} metrics[] = {
+
+		{ "wsrep_apply_oooe",                "operations",   DS_TYPE_DERIVE },
+		{ "wsrep_apply_oool",                "operations",   DS_TYPE_DERIVE },
+		{ "wsrep_causal_reads",              "operations",   DS_TYPE_DERIVE },
+		{ "wsrep_commit_oooe",               "operations",   DS_TYPE_DERIVE },
+		{ "wsrep_commit_oool",               "operations",   DS_TYPE_DERIVE },
+		{ "wsrep_flow_control_recv",         "operations",   DS_TYPE_DERIVE },
+		{ "wsrep_flow_control_sent",         "operations",   DS_TYPE_DERIVE },
+		{ "wsrep_flow_control_paused",       "operations",   DS_TYPE_DERIVE },
+		{ "wsrep_local_bf_aborts",           "operations",   DS_TYPE_DERIVE },
+		{ "wsrep_local_cert_failures",       "operations",   DS_TYPE_DERIVE },
+		{ "wsrep_local_commits",             "operations",   DS_TYPE_DERIVE },
+		{ "wsrep_local_replays",             "operations",   DS_TYPE_DERIVE },
+		{ "wsrep_received",                  "operations",   DS_TYPE_DERIVE },
+		{ "wsrep_replicated",                "operations",   DS_TYPE_DERIVE },
+
+		{ "wsrep_received_bytes",            "total_bytes",  DS_TYPE_DERIVE },
+		{ "wsrep_replicated_bytes",          "total_bytes",  DS_TYPE_DERIVE },
+
+		{ "wsrep_apply_window",              "gauge",        DS_TYPE_GAUGE },
+		{ "wsrep_commit_window",             "gauge",        DS_TYPE_GAUGE },
+
+		{ "wsrep_cluster_size",              "gauge",        DS_TYPE_GAUGE },
+		{ "wsrep_cert_deps_distance",        "gauge",        DS_TYPE_GAUGE },
+
+		{ "wsrep_local_recv_queue",          "queue_length", DS_TYPE_GAUGE },
+		{ "wsrep_local_send_queue",          "queue_length", DS_TYPE_GAUGE },
+
+		{ NULL,                              NULL,           0}
+
+	};
+
+	query = "SHOW GLOBAL STATUS LIKE 'wsrep_%'";
+
+	res = exec_query (con, query);
+	if (res == NULL)
+		return (-1);
+
+	row = mysql_fetch_row (res);
+	if (row == NULL)
+	{
+		ERROR ("mysql plugin: Failed to get wsrep statistics: "
+			"`%s' did not return any rows.", query);
+		mysql_free_result (res);
+		return (-1);
+	}
+
+	while ((row = mysql_fetch_row (res)))
+	{
+		int i;
+		char *key;
+		unsigned long long val;
+
+		key = row[0];
+		val = atoll (row[1]);
+
+		for (i = 0; metrics[i].key != NULL && strcmp(metrics[i].key, key) != 0; i++)
+			;
+
+		if (metrics[i].key == NULL)
+			continue;
+
+		switch (metrics[i].ds_type) {
+			case DS_TYPE_GAUGE:
+				gauge_submit(metrics[i].type, key, (gauge_t)val, db);
+				break;
+			case DS_TYPE_DERIVE:
+				derive_submit(metrics[i].type, key, (derive_t)val, db);
+				break;
+		}
+	}
+
+	mysql_free_result(res);
+	return (0);
+} /* mysql_read_wsrep_stats */
+
 static int mysql_read (user_data_t *ud)
 {
 	mysql_database_t *db;
-	MYSQL     *con;
-	MYSQL_RES *res;
-	MYSQL_ROW  row;
-	char      *query;
+	MYSQL      *con;
+	MYSQL_RES  *res;
+	MYSQL_ROW   row;
+	const char *query;
 
 	derive_t qcache_hits          = 0;
 	derive_t qcache_inserts       = 0;
@@ -712,7 +799,7 @@ static int mysql_read (user_data_t *ud)
 
 	unsigned long long traffic_incoming = 0ULL;
 	unsigned long long traffic_outgoing = 0ULL;
-    unsigned long mysql_version = 0ULL;
+	unsigned long mysql_version = 0ULL;
 
 	if ((ud == NULL) || (ud->data == NULL))
 	{
@@ -726,7 +813,7 @@ static int mysql_read (user_data_t *ud)
 	if ((con = getconnection (db)) == NULL)
 		return (-1);
 
-  mysql_version = mysql_get_server_version(con);
+	mysql_version = mysql_get_server_version(con);
 
 	query = "SHOW STATUS";
 	if (mysql_version >= 50002)
@@ -744,7 +831,7 @@ static int mysql_read (user_data_t *ud)
 		key = row[0];
 		val = atoll (row[1]);
 
-		if (strncmp (key, "Com_", 
+		if (strncmp (key, "Com_",
 			          strlen ("Com_")) == 0)
 		{
 			if (val == 0ULL)
@@ -752,18 +839,18 @@ static int mysql_read (user_data_t *ud)
 
 			/* Ignore `prepared statements' */
 			if (strncmp (key, "Com_stmt_", strlen ("Com_stmt_")) != 0)
-				counter_submit ("mysql_commands", 
-						key + strlen ("Com_"), 
+				counter_submit ("mysql_commands",
+						key + strlen ("Com_"),
 						val, db);
 		}
-		else if (strncmp (key, "Handler_", 
+		else if (strncmp (key, "Handler_",
 				        strlen ("Handler_")) == 0)
 		{
 			if (val == 0ULL)
 				continue;
 
-			counter_submit ("mysql_handler", 
-					key + strlen ("Handler_"), 
+			counter_submit ("mysql_handler",
+					key + strlen ("Handler_"),
 					val, db);
 		}
 		else if (strncmp (key, "Qcache_",
@@ -780,7 +867,7 @@ static int mysql_read (user_data_t *ud)
 			else if (strcmp (key, "Qcache_queries_in_cache") == 0)
 				qcache_queries_in_cache = (gauge_t) val;
 		}
-		else if (strncmp (key, "Bytes_", 
+		else if (strncmp (key, "Bytes_",
 				        strlen ("Bytes_")) == 0)
 		{
 			if (strcmp (key, "Bytes_received") == 0)
@@ -788,7 +875,7 @@ static int mysql_read (user_data_t *ud)
 			else if (strcmp (key, "Bytes_sent") == 0)
 				traffic_outgoing += val;
 		}
-		else if (strncmp (key, "Threads_", 
+		else if (strncmp (key, "Threads_",
        				        strlen ("Threads_")) == 0)
 		{
 			if (strcmp (key, "Threads_running") == 0)
@@ -832,6 +919,8 @@ static int mysql_read (user_data_t *ud)
 				counter_submit ("mysql_bpool_counters", "read_requests", val, db);
 			else if (strcmp (key, "Innodb_buffer_pool_reads") == 0)
 				counter_submit ("mysql_bpool_counters", "reads", val, db);
+			else if (strcmp (key, "Innodb_buffer_pool_wait_free") == 0)
+				counter_submit ("mysql_bpool_counters", "wait_free", val, db);
 			else if (strcmp (key, "Innodb_buffer_pool_write_requests") == 0)
 				counter_submit ("mysql_bpool_counters", "write_requests", val, db);
 			else if (strcmp (key, "Innodb_buffer_pool_bytes_data") == 0)
@@ -856,6 +945,8 @@ static int mysql_read (user_data_t *ud)
 				counter_submit ("mysql_innodb_dblwr", "writes", val, db);
 			else if (strcmp (key, "Innodb_dblwr_pages_written") == 0)
 				counter_submit ("mysql_innodb_dblwr", "written", val, db);
+			else if (strcmp (key, "Innodb_dblwr_page_size") == 0)
+				gauge_submit ("mysql_innodb_dblwr", "page_size", val, db);
 
 			/* log */
 			else if (strcmp (key, "Innodb_log_waits") == 0)
@@ -900,8 +991,19 @@ static int mysql_read (user_data_t *ud)
 		}
 		else if (strncmp (key, "Sort_", strlen ("Sort_")) == 0)
 		{
-			counter_submit ("mysql_sort", key + strlen ("Sort_"),
-					val, db);
+			if (strcmp (key, "Sort_merge_passes") == 0)
+				counter_submit ("mysql_sort_merge_passes", NULL, val, db);
+			else if (strcmp (key, "Sort_rows") == 0)
+				counter_submit ("mysql_sort_rows", NULL, val, db);
+			else if (strcmp (key, "Sort_range") == 0)
+				counter_submit ("mysql_sort", "range", val, db);
+			else if (strcmp (key, "Sort_scan") == 0)
+				counter_submit ("mysql_sort", "scan", val, db);
+
+		}
+		else if (strncmp (key, "Slow_queries", strlen ("Slow_queries")) == 0)
+		{
+			counter_submit ("mysql_slow_queries", NULL , val, db);
 		}
 	}
 	mysql_free_result (res); res = NULL;
@@ -940,7 +1042,7 @@ static int mysql_read (user_data_t *ud)
 	traffic_submit  (traffic_incoming, traffic_outgoing, db);
 
 	if (mysql_version >= 50600 && db->innodb_stats)
-        mysql_read_innodb_stats (db, con);
+		mysql_read_innodb_stats (db, con);
 
 	if (db->master_stats)
 		mysql_read_master_stats (db, con);
@@ -948,6 +1050,9 @@ static int mysql_read (user_data_t *ud)
 	if ((db->slave_stats) || (db->slave_notif))
 		mysql_read_slave_stats (db, con);
 
+	if (db->wsrep_stats)
+		mysql_read_wsrep_stats (db, con);
+
 	return (0);
 } /* int mysql_read */
 
