@@ -21,13 +21,14 @@
  * DEALINGS IN THE SOFTWARE.
  *
  * Authors:
- *   Sven Trenkel <collectd at semidefinite.de>  
+ *   Sven Trenkel <collectd at semidefinite.de>
  **/
 
 #include <Python.h>
 #include <structmember.h>
 
 #include "collectd.h"
+
 #include "common.h"
 
 #include "cpython.h"
@@ -45,7 +46,7 @@ static PyObject *cpy_common_repr(PyObject *s) {
 	static PyObject *l_type = NULL, *l_type_instance = NULL, *l_plugin = NULL, *l_plugin_instance = NULL;
 	static PyObject *l_host = NULL, *l_time = NULL;
 	PluginData *self = (PluginData *) s;
-	
+
 	if (l_type == NULL)
 		l_type = cpy_string_to_unicode_or_bytes("(type=");
 	if (l_type_instance == NULL)
@@ -58,10 +59,10 @@ static PyObject *cpy_common_repr(PyObject *s) {
 		l_host = cpy_string_to_unicode_or_bytes(",host=");
 	if (l_time == NULL)
 		l_time = cpy_string_to_unicode_or_bytes(",time=");
-	
+
 	if (!l_type || !l_type_instance || !l_plugin || !l_plugin_instance || !l_host || !l_time)
 		return NULL;
-	
+
 	ret = cpy_string_to_unicode_or_bytes(s->ob_type->tp_name);
 
 	CPY_STRCAT(&ret, l_type);
@@ -106,19 +107,19 @@ static PyObject *cpy_common_repr(PyObject *s) {
 	return ret;
 }
 
-static char time_doc[] = "This is the Unix timestap of the time this value was read.\n"
+static char time_doc[] = "This is the Unix timestamp of the time this value was read.\n"
 		"For dispatching values this can be set to 0 which means \"now\".\n"
 		"This means the time the value is actually dispatched, not the time\n"
 		"it was set to 0.";
 
 static char host_doc[] = "The hostname of the host this value was read from.\n"
 		"For dispatching this can be set to an empty string which means\n"
-		"the local hostname as defined in the collectd.conf.";
+		"the local hostname as defined in collectd.conf.";
 
 static char type_doc[] = "The type of this value. This type has to be defined\n"
-		"in your types.db. Attempting to set it to any other value will\n"
-		"raise a TypeError exception.\n"
-		"Assigning a type is mandetory, calling dispatch without doing\n"
+		"in the types.db file. Attempting to set it to any other value\n"
+		"will raise a TypeError exception.\n"
+		"Assigning a type is mandatory, calling dispatch without doing\n"
 		"so will raise a RuntimeError exception.";
 
 static char type_instance_doc[] = "";
@@ -129,16 +130,16 @@ static char plugin_doc[] = "The name of the plugin that read the data. Setting t
 static char plugin_instance_doc[] = "";
 
 static char PluginData_doc[] = "This is an internal class that is the base for Values\n"
-		"and Notification. It is pretty useless by itself and was therefore not\n"
+		"and Notification. It is pretty useless by itself and is therefore not\n"
 		"exported to the collectd module.";
 
 static PyObject *PluginData_new(PyTypeObject *type, PyObject *args, PyObject *kwds) {
 	PluginData *self;
-	
+
 	self = (PluginData *) type->tp_alloc(type, 0);
 	if (self == NULL)
 		return NULL;
-	
+
 	self->time = 0;
 	self->host[0] = 0;
 	self->plugin[0] = 0;
@@ -154,11 +155,11 @@ static int PluginData_init(PyObject *s, PyObject *args, PyObject *kwds) {
 	char *type = NULL, *plugin_instance = NULL, *type_instance = NULL, *plugin = NULL, *host = NULL;
 	static char *kwlist[] = {"type", "plugin_instance", "type_instance",
 			"plugin", "host", "time", NULL};
-	
+
 	if (!PyArg_ParseTupleAndKeywords(args, kwds, "|etetetetetd", kwlist, NULL, &type,
 			NULL, &plugin_instance, NULL, &type_instance, NULL, &plugin, NULL, &host, &time))
 		return -1;
-	
+
 	if (type && plugin_get_ds(type) == NULL) {
 		PyErr_Format(PyExc_TypeError, "Dataset %s not found", type);
 		FreeAll();
@@ -180,13 +181,13 @@ static int PluginData_init(PyObject *s, PyObject *args, PyObject *kwds) {
 static PyObject *PluginData_repr(PyObject *s) {
 	PyObject *ret;
 	static PyObject *l_closing = NULL;
-	
+
 	if (l_closing == NULL)
 		l_closing = cpy_string_to_unicode_or_bytes(")");
-	
+
 	if (l_closing == NULL)
 		return NULL;
-	
+
 	ret = cpy_common_repr(s);
 	CPY_STRCAT(&ret, l_closing);
 	return ret;
@@ -199,14 +200,14 @@ static PyMemberDef PluginData_members[] = {
 
 static PyObject *PluginData_getstring(PyObject *self, void *data) {
 	const char *value = ((char *) self) + (intptr_t) data;
-	
+
 	return cpy_string_to_unicode_or_bytes(value);
 }
 
 static int PluginData_setstring(PyObject *self, PyObject *value, void *data) {
 	char *old;
 	const char *new;
-	
+
 	if (value == NULL) {
 		PyErr_SetString(PyExc_TypeError, "Cannot delete this attribute");
 		return -1;
@@ -226,7 +227,7 @@ static int PluginData_setstring(PyObject *self, PyObject *value, void *data) {
 static int PluginData_settype(PyObject *self, PyObject *value, void *data) {
 	char *old;
 	const char *new;
-	
+
 	if (value == NULL) {
 		PyErr_SetString(PyExc_TypeError, "Cannot delete this attribute");
 		return -1;
@@ -312,8 +313,8 @@ static char interval_doc[] = "The interval is the timespan in seconds between tw
 static char values_doc[] = "These are the actual values that get dispatched to collectd.\n"
 		"It has to be a sequence (a tuple or list) of numbers.\n"
 		"The size of the sequence and the type of its content depend on the type\n"
-		"member your types.db file. For more information on this read the types.db\n"
-		"man page.\n"
+		"member in the types.db file. For more information on this read the\n"
+		"types.db man page.\n"
 		"\n"
 		"If the sequence does not have the correct size upon dispatch a RuntimeError\n"
 		"exception will be raised. If the content of the sequence is not a number,\n"
@@ -322,7 +323,7 @@ static char values_doc[] = "These are the actual values that get dispatched to c
 static char meta_doc[] = "These are the meta data for this Value object.\n"
 		"It has to be a dictionary of numbers, strings or bools. All keys must be\n"
 		"strings. int and long objects will be dispatched as signed integers unless\n"
-		"they are between 2**63 and 2**64-1, which will result in a unsigned integer.\n"
+		"they are between 2**63 and 2**64-1, which will result in an unsigned integer.\n"
 		"You can force one of these storage classes by using the classes\n"
 		"collectd.Signed and collectd.Unsigned. A meta object received by a write\n"
 		"callback will always contain Signed or Unsigned objects.";
@@ -340,7 +341,7 @@ static char dispatch_doc[] = "dispatch([type][, values][, plugin_instance][, typ
 static char write_doc[] = "write([destination][, type][, values][, plugin_instance][, type_instance]"
 		"[, plugin][, host][, time][, interval]) -> None.  Dispatch a value list.\n"
 		"\n"
-		"Write this instance to a single plugin or all plugins if 'destination' is obmitted.\n"
+		"Write this instance to a single plugin or all plugins if 'destination' is omitted.\n"
 		"This will bypass the main collectd process and all filtering and caching.\n"
 		"Other than that it works similar to 'dispatch'. In most cases 'dispatch' should be\n"
 		"used instead of 'write'.\n";
@@ -349,11 +350,11 @@ static char Values_doc[] = "A Values object used for dispatching values to colle
 
 static PyObject *Values_new(PyTypeObject *type, PyObject *args, PyObject *kwds) {
 	Values *self;
-	
+
 	self = (Values *) PluginData_new(type, args, kwds);
 	if (self == NULL)
 		return NULL;
-	
+
 	self->values = PyList_New(0);
 	self->meta = PyDict_New();
 	self->interval = 0;
@@ -367,12 +368,12 @@ static int Values_init(PyObject *s, PyObject *args, PyObject *kwds) {
 	char *type = NULL, *plugin_instance = NULL, *type_instance = NULL, *plugin = NULL, *host = NULL;
 	static char *kwlist[] = {"type", "values", "plugin_instance", "type_instance",
 			"plugin", "host", "time", "interval", "meta", NULL};
-	
+
 	if (!PyArg_ParseTupleAndKeywords(args, kwds, "|etOetetetetddO", kwlist,
 			NULL, &type, &values, NULL, &plugin_instance, NULL, &type_instance,
 			NULL, &plugin, NULL, &host, &time, &interval, &meta))
 		return -1;
-	
+
 	if (type && plugin_get_ds(type) == NULL) {
 		PyErr_Format(PyExc_TypeError, "Dataset %s not found", type);
 		FreeAll();
@@ -394,18 +395,18 @@ static int Values_init(PyObject *s, PyObject *args, PyObject *kwds) {
 	} else {
 		Py_INCREF(values);
 	}
-	
+
 	if (meta == NULL) {
 		meta = PyDict_New();
 		PyErr_Clear();
 	} else {
 		Py_INCREF(meta);
 	}
-	
+
 	tmp = self->values;
 	self->values = values;
 	Py_XDECREF(tmp);
-	
+
 	tmp = self->meta;
 	self->meta = meta;
 	Py_XDECREF(tmp);
@@ -415,10 +416,10 @@ static int Values_init(PyObject *s, PyObject *args, PyObject *kwds) {
 }
 
 static meta_data_t *cpy_build_meta(PyObject *meta) {
-	int i, s;
+	int s;
 	meta_data_t *m = NULL;
 	PyObject *l;
-	
+
 	if ((meta == NULL) || (meta == Py_None))
 		return NULL;
 
@@ -434,10 +435,10 @@ static meta_data_t *cpy_build_meta(PyObject *meta) {
 	}
 
 	m = meta_data_create();
-	for (i = 0; i < s; ++i) {
+	for (int i = 0; i < s; ++i) {
 		const char *string, *keystring;
 		PyObject *key, *value, *item, *tmp;
-		
+
 		item = PyList_GET_ITEM(l, i);
 		key = PyTuple_GET_ITEM(item, 0);
 		Py_INCREF(key);
@@ -502,15 +503,15 @@ static meta_data_t *cpy_build_meta(PyObject *meta) {
 }
 
 static PyObject *Values_dispatch(Values *self, PyObject *args, PyObject *kwds) {
-	int i, ret;
+	int ret;
 	const data_set_t *ds;
-	int size;
+	size_t size;
 	value_t *value;
 	value_list_t value_list = VALUE_LIST_INIT;
 	PyObject *values = self->values, *meta = self->meta;
 	double time = self->data.time, interval = self->interval;
 	char *host = NULL, *plugin = NULL, *plugin_instance = NULL, *type = NULL, *type_instance = NULL;
-	
+
 	static char *kwlist[] = {"type", "values", "plugin_instance", "type_instance",
 			"plugin", "host", "time", "interval", "meta", NULL};
 	if (!PyArg_ParseTupleAndKeywords(args, kwds, "|etOetetetetddO", kwlist,
@@ -542,15 +543,15 @@ static PyObject *Values_dispatch(Values *self, PyObject *args, PyObject *kwds) {
 		PyErr_Format(PyExc_TypeError, "meta must be a dict");
 		return NULL;
 	}
-	size = (int) PySequence_Length(values);
+	size = (size_t) PySequence_Length(values);
 	if (size != ds->ds_num) {
-		PyErr_Format(PyExc_RuntimeError, "type %s needs %d values, got %i", value_list.type, ds->ds_num, size);
+		PyErr_Format(PyExc_RuntimeError, "type %s needs %zu values, got %zu", value_list.type, ds->ds_num, size);
 		return NULL;
 	}
-	value = malloc(size * sizeof(*value));
-	for (i = 0; i < size; ++i) {
+	value = calloc(size, sizeof(*value));
+	for (size_t i = 0; i < size; ++i) {
 		PyObject *item, *num;
-		item = PySequence_Fast_GET_ITEM(values, i); /* Borrowed reference. */
+		item = PySequence_Fast_GET_ITEM(values, (int) i); /* Borrowed reference. */
 		switch (ds->ds[i].type) {
 		case DS_TYPE_COUNTER:
 			num = PyNumber_Long(item); /* New reference. */
@@ -616,15 +617,15 @@ static PyObject *Values_dispatch(Values *self, PyObject *args, PyObject *kwds) {
 }
 
 static PyObject *Values_write(Values *self, PyObject *args, PyObject *kwds) {
-	int i, ret;
+	int ret;
 	const data_set_t *ds;
-	int size;
+	size_t size;
 	value_t *value;
 	value_list_t value_list = VALUE_LIST_INIT;
 	PyObject *values = self->values, *meta = self->meta;
 	double time = self->data.time, interval = self->interval;
 	char *host = NULL, *plugin = NULL, *plugin_instance = NULL, *type = NULL, *type_instance = NULL, *dest = NULL;
-	
+
 	static char *kwlist[] = {"destination", "type", "values", "plugin_instance", "type_instance",
 			"plugin", "host", "time", "interval", "meta", NULL};
 	if (!PyArg_ParseTupleAndKeywords(args, kwds, "et|etOetetetetdiO", kwlist, NULL, &dest,
@@ -651,13 +652,13 @@ static PyObject *Values_write(Values *self, PyObject *args, PyObject *kwds) {
 		PyErr_Format(PyExc_TypeError, "values must be list or tuple");
 		return NULL;
 	}
-	size = (int) PySequence_Length(values);
+	size = (size_t) PySequence_Length(values);
 	if (size != ds->ds_num) {
-		PyErr_Format(PyExc_RuntimeError, "type %s needs %d values, got %i", value_list.type, ds->ds_num, size);
+		PyErr_Format(PyExc_RuntimeError, "type %s needs %zu values, got %zu", value_list.type, ds->ds_num, size);
 		return NULL;
 	}
-	value = malloc(size * sizeof(*value));
-	for (i = 0; i < size; ++i) {
+	value = calloc(size, sizeof(*value));
+	for (size_t i = 0; i < size; ++i) {
 		PyObject *item, *num;
 		item = PySequence_Fast_GET_ITEM(values, i); /* Borrowed reference. */
 		switch (ds->ds[i].type) {
@@ -707,7 +708,7 @@ static PyObject *Values_write(Values *self, PyObject *args, PyObject *kwds) {
 	value_list.values_len = size;
 	value_list.time = DOUBLE_TO_CDTIME_T(time);
 	value_list.interval = DOUBLE_TO_CDTIME_T(interval);
-	value_list.meta = cpy_build_meta(meta);;
+	value_list.meta = cpy_build_meta(meta);
 	if (value_list.host[0] == 0)
 		sstrncpy(value_list.host, hostname_g, sizeof(value_list.host));
 	if (value_list.plugin[0] == 0)
@@ -728,7 +729,7 @@ static PyObject *Values_repr(PyObject *s) {
 	PyObject *ret, *tmp;
 	static PyObject *l_interval = NULL, *l_values = NULL, *l_meta = NULL, *l_closing = NULL;
 	Values *self = (Values *) s;
-	
+
 	if (l_interval == NULL)
 		l_interval = cpy_string_to_unicode_or_bytes(",interval=");
 	if (l_values == NULL)
@@ -737,10 +738,10 @@ static PyObject *Values_repr(PyObject *s) {
 		l_meta = cpy_string_to_unicode_or_bytes(",meta=");
 	if (l_closing == NULL)
 		l_closing = cpy_string_to_unicode_or_bytes(")");
-	
+
 	if (l_interval == NULL || l_values == NULL || l_meta == NULL || l_closing == NULL)
 		return NULL;
-	
+
 	ret = cpy_common_repr(s);
 	if (self->interval != 0) {
 		CPY_STRCAT(&ret, l_interval);
@@ -854,12 +855,12 @@ static int Notification_init(PyObject *s, PyObject *args, PyObject *kwds) {
 	char *type = NULL, *plugin_instance = NULL, *type_instance = NULL, *plugin = NULL, *host = NULL;
 	static char *kwlist[] = {"type", "message", "plugin_instance", "type_instance",
 			"plugin", "host", "time", "severity", NULL};
-	
+
 	if (!PyArg_ParseTupleAndKeywords(args, kwds, "|etetetetetetdi", kwlist,
 			NULL, &type, NULL, &message, NULL, &plugin_instance, NULL, &type_instance,
 			NULL, &plugin, NULL, &host, &time, &severity))
 		return -1;
-	
+
 	if (type && plugin_get_ds(type) == NULL) {
 		PyErr_Format(PyExc_TypeError, "Dataset %s not found", type);
 		FreeAll();
@@ -889,7 +890,7 @@ static PyObject *Notification_dispatch(Notification *self, PyObject *args, PyObj
 	int severity = self->severity;
 	char *host = NULL, *plugin = NULL, *plugin_instance = NULL, *type = NULL, *type_instance = NULL;
 	char *message = NULL;
-	
+
 	static char *kwlist[] = {"type", "message", "plugin_instance", "type_instance",
 			"plugin", "host", "time", "severity", NULL};
 	if (!PyArg_ParseTupleAndKeywords(args, kwds, "|etetetetetetdi", kwlist,
@@ -937,11 +938,11 @@ static PyObject *Notification_dispatch(Notification *self, PyObject *args, PyObj
 
 static PyObject *Notification_new(PyTypeObject *type, PyObject *args, PyObject *kwds) {
 	Notification *self;
-	
+
 	self = (Notification *) PluginData_new(type, args, kwds);
 	if (self == NULL)
 		return NULL;
-	
+
 	self->message[0] = 0;
 	self->severity = 0;
 	return (PyObject *) self;
@@ -950,7 +951,7 @@ static PyObject *Notification_new(PyTypeObject *type, PyObject *args, PyObject *
 static int Notification_setstring(PyObject *self, PyObject *value, void *data) {
 	char *old;
 	const char *new;
-	
+
 	if (value == NULL) {
 		PyErr_SetString(PyExc_TypeError, "Cannot delete this attribute");
 		return -1;
@@ -971,17 +972,17 @@ static PyObject *Notification_repr(PyObject *s) {
 	PyObject *ret, *tmp;
 	static PyObject *l_severity = NULL, *l_message = NULL, *l_closing = NULL;
 	Notification *self = (Notification *) s;
-	
+
 	if (l_severity == NULL)
 		l_severity = cpy_string_to_unicode_or_bytes(",severity=");
 	if (l_message == NULL)
 		l_message = cpy_string_to_unicode_or_bytes(",message=");
 	if (l_closing == NULL)
 		l_closing = cpy_string_to_unicode_or_bytes(")");
-	
+
 	if (l_severity == NULL || l_message == NULL || l_closing == NULL)
 		return NULL;
-	
+
 	ret = cpy_common_repr(s);
 	if (self->severity != 0) {
 		CPY_STRCAT(&ret, l_severity);
