@@ -24,6 +24,7 @@
  **/
 
 #include "collectd.h"
+
 #include "plugin.h"
 #include "common.h"
 #include "utils_avltree.h"
@@ -32,10 +33,6 @@
 
 #include <rrd.h>
 
-#if HAVE_PTHREAD_H
-# include <pthread.h>
-#endif
-
 /*
  * Private types
  */
@@ -166,7 +163,7 @@ static int srrd_update (char *filename, char *template,
 	assert (template == NULL);
 
 	new_argc = 2 + argc;
-	new_argv = (char **) malloc ((new_argc + 1) * sizeof (char *));
+	new_argv = malloc ((new_argc + 1) * sizeof (*new_argv));
 	if (new_argv == NULL)
 	{
 		ERROR ("rrdtool plugin: malloc failed.");
@@ -204,7 +201,6 @@ static int value_list_to_string_multiple (char *buffer, int buffer_len,
 	int offset;
 	int status;
 	time_t tt;
-	int i;
 
 	memset (buffer, '\0', buffer_len);
 
@@ -214,7 +210,7 @@ static int value_list_to_string_multiple (char *buffer, int buffer_len,
 		return (-1);
 	offset = status;
 
-	for (i = 0; i < ds->ds_num; i++)
+	for (size_t i = 0; i < ds->ds_num; i++)
 	{
 		if ((ds->ds[i].type != DS_TYPE_COUNTER)
 				&& (ds->ds[i].type != DS_TYPE_GAUGE)
@@ -335,7 +331,6 @@ static void *rrd_queue_thread (void __attribute__((unused)) *data)
 		char **values;
 		int    values_num;
 		int    status;
-		int    i;
 
 		values = NULL;
 		values_num = 0;
@@ -416,7 +411,7 @@ static void *rrd_queue_thread (void __attribute__((unused)) *data)
 		pthread_mutex_unlock (&queue_lock);
 
 		/* We now need the cache lock so the entry isn't updated while
-		 * we make a copy of it's values */
+		 * we make a copy of its values */
 		pthread_mutex_lock (&cache_lock);
 
 		status = c_avl_get (cache, queue_entry->filename,
@@ -442,7 +437,7 @@ static void *rrd_queue_thread (void __attribute__((unused)) *data)
 		}
 
 		/* Update `tv_next_update' */
-		if (write_rate > 0.0) 
+		if (write_rate > 0.0)
                 {
                   gettimeofday (&tv_now, /* timezone = */ NULL);
                   tv_next_update.tv_sec = tv_now.tv_sec;
@@ -462,7 +457,7 @@ static void *rrd_queue_thread (void __attribute__((unused)) *data)
 				values_num, (values_num == 1) ? "" : "s",
 				queue_entry->filename);
 
-		for (i = 0; i < values_num; i++)
+		for (int i = 0; i < values_num; i++)
 		{
 			sfree (values[i]);
 		}
@@ -480,7 +475,7 @@ static int rrd_queue_enqueue (const char *filename,
 {
   rrd_queue_t *queue_entry;
 
-  queue_entry = (rrd_queue_t *) malloc (sizeof (rrd_queue_t));
+  queue_entry = malloc (sizeof (*queue_entry));
   if (queue_entry == NULL)
     return (-1);
 
@@ -522,7 +517,7 @@ static int rrd_queue_dequeue (const char *filename,
   {
     if (strcmp (this->filename, filename) == 0)
       break;
-    
+
     prev = this;
     this = this->next;
   }
@@ -560,7 +555,6 @@ static void rrd_cache_flush (cdtime_t timeout)
 
 	char *key;
 	c_avl_iterator_t *iter;
-	int i;
 
 	DEBUG ("rrdtool plugin: Flushing cache, timeout = %.3f",
 			CDTIME_T_TO_DOUBLE (timeout));
@@ -588,7 +582,7 @@ static void rrd_cache_flush (cdtime_t timeout)
 		}
 		else /* ancient and no values -> waste of memory */
 		{
-			char **tmp = (char **) realloc ((void *) keys,
+			char **tmp = realloc (keys,
 					(keys_num + 1) * sizeof (char *));
 			if (tmp == NULL)
 			{
@@ -607,8 +601,8 @@ static void rrd_cache_flush (cdtime_t timeout)
 		}
 	} /* while (c_avl_iterator_next) */
 	c_avl_iterator_destroy (iter);
-	
-	for (i = 0; i < keys_num; i++)
+
+	for (int i = 0; i < keys_num; i++)
 	{
 		if (c_avl_remove (cache, keys[i], (void *) &key, (void *) &rc) != 0)
 		{
@@ -692,7 +686,7 @@ static int64_t rrd_get_random_variation (void)
   long min;
   long max;
 
-  if (random_timeout <= 0)
+  if (random_timeout == 0)
     return (0);
 
   /* Assure that "cache_timeout + random_variation" is never negative. */
@@ -756,7 +750,7 @@ static int rrd_cache_insert (const char *filename,
 		return (-1);
 	}
 
-	values_new = (char **) realloc ((void *) rc->values,
+	values_new = realloc ((void *) rc->values,
 			(rc->values_num + 1) * sizeof (char *));
 	if (values_new == NULL)
 	{
@@ -860,7 +854,6 @@ static int rrd_cache_destroy (void) /* {{{ */
   while (c_avl_pick (cache, &key, &value) == 0)
   {
     rrd_cache_t *rc;
-    int i;
 
     sfree (key);
     key = NULL;
@@ -871,7 +864,7 @@ static int rrd_cache_destroy (void) /* {{{ */
     if (rc->values_num > 0)
       non_empty++;
 
-    for (i = 0; i < rc->values_num; i++)
+    for (int i = 0; i < rc->values_num; i++)
       sfree (rc->values[i]);
     sfree (rc->values);
     sfree (rc);
@@ -1087,7 +1080,7 @@ static int rrd_config (const char *key, const char *value)
 		while ((ptr = strtok_r (dummy, ", \t", &saveptr)) != NULL)
 		{
 			dummy = NULL;
-			
+
 			tmp_alloc = realloc (rrdcreate_config.timespans,
 					sizeof (int) * (rrdcreate_config.timespans_num + 1));
 			if (tmp_alloc == NULL)
