@@ -27,6 +27,7 @@
  **/
 
 #include "collectd.h"
+
 #include "common.h"
 #include "plugin.h"
 
@@ -138,7 +139,7 @@ static int vserver_read (void)
 	if (proc == NULL)
 	{
 		char errbuf[1024];
-		ERROR ("vserver plugin: fopen (%s): %s", PROCDIR, 
+		ERROR ("vserver plugin: fopen (%s): %s", PROCDIR,
 				sstrerror (errno, errbuf, sizeof (errbuf)));
 		return (-1);
 	}
@@ -178,7 +179,7 @@ static int vserver_read (void)
 		len = ssnprintf (file, sizeof (file), PROCDIR "/%s", dent->d_name);
 		if ((len < 0) || (len >= BUFSIZE))
 			continue;
-		
+
 		status = stat (file, &statbuf);
 		if (status != 0)
 		{
@@ -187,7 +188,7 @@ static int vserver_read (void)
 					file, sstrerror (errno, errbuf, sizeof (errbuf)));
 			continue;
 		}
-		
+
 		if (!S_ISDIR (statbuf.st_mode))
 			continue;
 
@@ -208,7 +209,7 @@ static int vserver_read (void)
 		{
 			derive_t rx;
 			derive_t tx;
-			char *type_instance;
+			const char *type_instance;
 
 			if (strsplit (buffer, cols, 4) < 4)
 				continue;
@@ -258,7 +259,7 @@ static int vserver_read (void)
 
 			if (2 == n)
 			{
-				char   *type_instance;
+				const char *type_instance;
 				gauge_t value;
 
 				if (0 == strcmp (cols[0], "nr_threads:"))
@@ -307,8 +308,8 @@ static int vserver_read (void)
 
 		while ((fh != NULL) && (NULL != fgets (buffer, BUFSIZE, fh)))
 		{
-			char *type = "vs_memory";
-			char *type_instance;
+			const char *type = "vs_memory";
+			const char *type_instance;
 			gauge_t value;
 
 			if (strsplit (buffer, cols, 2) < 2)
